/*****************************************************************************

		Copyright (c) 2000 Torsten Kurbad Technische Universitt Ilmenau

 Project:  VORFAHRT
 FileName: VORFAHRT.PRO
 Purpose: Vorfahrtsregeln in Prolog
 Written by: Torsten Kurbad
 Comments: bugfix by Thomas 'Silvo' Volkert, 05 / 2003
******************************************************************************/

include "vorfahrt.inc"
include "vorfahrt.con"
include "vorfahrt.pre"
include "hlptopic.con"
include "testenv.pre"
include "new_ref.pre"

database - modus

run_modus(integer)

database - lauf

run_num(integer)

database - windows

window_handle(string,window)
win_content(window,picture)

predicates

length(vorrang,integer)
length_rek(vorrang,integer,integer)
uebrig(integer)
vergleiche(vorrang,vorrang)
teste_loesung(kreuzung,integer,vorrang,vorrang)
start(integer,integer)
grafik(kreuzung,vorrang,vorrang)
finde_hstr(kreuzung,vorrang)
haupt_pos(l_str,vorrang)
haupt_pos_rek(integer,l_str,vorrang,vorrang)
ersetze_alle_richtg(kreuzung,vorrang)
ersetze_alle_richtg_rek(l_fzg,vorrang,vorrang)

clauses

haupt_pos(Strassen,Positionen):-
	haupt_pos_rek(1,Strassen,[],Positionen).

haupt_pos_rek(_,[],Positionen,Positionen):-!.
haupt_pos_rek(Zaehl,[h|Rest],Positionen1,Positionen):-!,
	Positionen2=[Zaehl|Positionen1],
	Zaehl1=Zaehl+1,
	haupt_pos_rek(Zaehl1,Rest,Positionen2,Positionen).
haupt_pos_rek(Zaehl,[_|Rest],Positionen1,Positionen):-!,
	Zaehl1=Zaehl+1,
	haupt_pos_rek(Zaehl1,Rest,Positionen1,Positionen).

finde_hstr([r(_)|_],[]):-!.
finde_hstr([a(_,Str1),a(_,Str2),a(_,Str3),a(_,Str4)],Liste):-!,
	haupt_pos([Str1,Str2,Str3,Str4],Liste).
finde_hstr([h(_,Str1),h(_,Str2),h(_,Str3),h(_,Str4)],Liste):-!,
	haupt_pos([Str1,Str2,Str3,Str4],Liste).

ersetze_alle_richtg([r(R1),r(R2),r(R3),r(R4)],Richtungen):-!,
	ersetze_alle_richtg_rek([R4,R3,R2,R1],[],Richtungen).
ersetze_alle_richtg([a(R1,_),a(R2,_),a(R3,_),a(R4,_)],Richtungen):-!,
	ersetze_alle_richtg_rek([R4,R3,R2,R1],[],Richtungen).
ersetze_alle_richtg([h(R1,_),h(R2,_),h(R3,_),h(R4,_)],Richtungen):-
	ersetze_alle_richtg_rek([R4,R3,R2,R1],[],Richtungen).

ersetze_alle_richtg_rek([],Richtungen,Richtungen):-!.
ersetze_alle_richtg_rek([R|Rest],Richtgn,Richtungen):-
	num2fzg(Rn,R),
	ersetze_alle_richtg_rek(Rest,[Rn|Richtgn],Richtungen).

length(Liste,Laenge):-
	length_rek(Liste,0,Laenge).

length_rek([],Laenge,Laenge):-!.
length_rek([_|R],Laenge1,Laenge):-
	Laenge2=Laenge1+1,
	length_rek(R,Laenge2,Laenge).

uebrig(Uebrig):-
	run_num(Runs),
	Runs>1,!,
	write("Es folgen ",Uebrig," Durchlufe.\n---\n\n").
uebrig(_):-
	write("---\n\n").

vergleiche([],[]):-!.
vergleiche([K1|R1],[K1|R2]):-!,
	vergleiche(R1,R2).
vergleiche(_,_):-!,fail.

teste_loesung(_,Uebrig,Referenz,Reihenfolge):-
	length(Reihenfolge,Laenge),
	Laenge<>4,!,
	sound(80,100),
	Win=msg_GetWin(),
	win_SetForeColor(Win,0x0000e0),
	write("FEHLER: Ergebnis hat ",Laenge," statt 4 Elemente!\n"),
	write("Richtige Lsung: ",Referenz,"\n"),
	write("Ihr Ergebnis: ",Reihenfolge,"\n"),
	win_SetForeColor(Win,0x000000),
	uebrig(Uebrig).

teste_loesung(Testsit,Uebrig,Referenz,Reihenfolge):-
	vergleiche(Referenz,Reihenfolge),!,
	write("Ihr Ergebnis ",Reihenfolge," ist richtig!\n"),
	uebrig(Uebrig),
	grafik(Testsit,Referenz,Reihenfolge).

teste_loesung(Testsit,Uebrig,Referenz,Reihenfolge):-
	sound(50,150),
	write("FEHLER: Falsche Rangfolge\n"),
	write("Richtige Lsung: ",Referenz,"\n"),
	write("Ihr Ergebnis: ",Reihenfolge,"\n"),
	uebrig(Uebrig),
	grafik(Testsit,Referenz,Reihenfolge).

start(_,0):-!.
start(Modus,Anzahl):-!,
	generiere(Modus,Testsit),
	write("Situation: ",Testsit,"\n\n"),
	referenz(Testsit,Referenz),
	vorfahrt(Testsit,Reihenfolge),
	Anzahl1=Anzahl-1,
	teste_loesung(Testsit,Anzahl1,Referenz,Reihenfolge),
	start(Modus,Anzahl1).

predicates

garb_coll(window)
delay(vorrang,integer)
kreuzg_redraw(window)
kreuzg_background(window)
kreuzg_vzeichen(window,vorrang)
kreuzg_vzeichen_rek(integer,window,vorrang,picture,picture)
kreuzg_autos(window,vorrang,vorrang,vorrang,vorrang,vorrang)
kreuzg_autos_rek(window,vorrang,vorrang,vorrang,vorrang,vorrang,integer)
kreuzg_autos_start(window,vorrang,vorrang,vorrang,vorrang)
kreuzg_autos_richtg(window,integer,vorrang,long)
richtg_eckpunkte(integer,integer,pntlist)
kreuzg_autos_start_rek(window,integer,vorrang)
find_loc_vz(integer,integer,integer,integer,integer)
find_rot_vz(integer,integer)
grafik_position(integer,integer,integer,integer,integer,integer,integer)
pfeil_eckpunkte(integer,pnt,pnt,pnt,pnt)
erzeuge_graf_listen(vorrang,vorrang,vorrang,vorrang,vorrang)
erzeuge_graf_listen_rek(integer,vorrang,vorrang,vorrang,vorrang,vorrang,vorrang,vorrang,vorrang)
lade_bild(integer,integer,picture)

clauses

garb_coll(Win):-
	win_content(Win,PICT),!,
	pict_Destroy(PICT).
garb_coll(_).

delay([],_):-!.
delay(_,CSec):-
	sleep(CSec).

kreuzg_redraw(Win):-
	win_content(Win,KREUZGPICT),!,
	_RCT=win_GetClientRect(Win),
	pict_Draw(Win,KREUZGPICT,_RCT,_RCT,rop_SrcCopy),!.
kreuzg_redraw(Win):-
	kreuzg_background(Win).

%grafik_position(Richtg,Typ,Stelle,PosX,PosY,RotWinkel).
grafik_position(1,1,2,145,145,pictRotate_180deg,2):-!.
grafik_position(2,1,2,260,145,pictRotate_90deg,2):-!.
grafik_position(3,1,2,260,255,pictRotate_copy,2):-!.
grafik_position(4,1,2,145,255,pictRotate_270deg,2):-!.
grafik_position(1,2,2,175,200,pictRotate_180deg,1):-!.
grafik_position(2,2,2,205,175,pictRotate_90deg,1):-!.
grafik_position(3,2,2,250,205,pictRotate_copy,1):-!.
grafik_position(4,2,2,200,250,pictRotate_270deg,1):-!.
grafik_position(1,3,2,225,225,pictRotate_270deg,2):-!.
grafik_position(2,3,2,175,225,pictRotate_180deg,2):-!.
grafik_position(3,3,2,175,175,pictRotate_90deg,2):-!.
grafik_position(4,3,2,225,175,pictRotate_copy,2):-!.
grafik_position(1,1,3,70,175,pictRotate_90deg,1):-!.
grafik_position(2,1,3,250,70,pictRotate_copy,1):-!.
grafik_position(3,1,3,330,250,pictRotate_270deg,1):-!.
grafik_position(4,1,3,175,330,pictRotate_180deg,1):-!.
grafik_position(1,2,3,175,330,pictRotate_180deg,1):-!.
grafik_position(2,2,3,70,175,pictRotate_90deg,1):-!.
grafik_position(3,2,3,250,70,pictRotate_copy,1):-!.
grafik_position(4,2,3,330,250,pictRotate_270deg,1):-!.
grafik_position(1,3,3,330,250,pictRotate_270deg,1):-!.
grafik_position(2,3,3,175,330,pictRotate_180deg,1):-!.
grafik_position(3,3,3,70,175,pictRotate_90deg,1):-!.
grafik_position(4,3,3,250,70,pictRotate_copy,1):-!.
grafik_position(1,_,_,175,70,pictRotate_180deg,1):-!.
grafik_position(2,_,_,330,175,pictRotate_90deg,1):-!.
grafik_position(3,_,_,250,330,pictRotate_copy,1):-!.
grafik_position(4,_,_,70,250,pictRotate_270deg,1).

find_rot_vz(1,pictRotate_180deg):-!.
find_rot_vz(2,pictRotate_90deg):-!.
find_rot_vz(3,pictRotate_copy):-!.
find_rot_vz(4,pictRotate_270deg).

find_loc_vz(1,120,71,149,100):-!.
find_loc_vz(2,350,120,379,149):-!.
find_loc_vz(3,300,350,329,379):-!.
find_loc_vz(4,71,300,100,329).

pfeil_eckpunkte(1,pnt(263,110),pnt(255,110),pnt(263,90),pnt(271,110)):-!.
pfeil_eckpunkte(2,pnt(335,263),pnt(335,255),pnt(355,263),pnt(335,271)):-!.
pfeil_eckpunkte(3,pnt(187,335),pnt(179,335),pnt(187,355),pnt(195,335)):-!.
pfeil_eckpunkte(4,pnt(110,187),pnt(110,179),pnt(90,187),pnt(110,195)).

richtg_eckpunkte(1,1,[pnt(187,120),pnt(187,153),pnt(148,187),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(4,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(1,2,[pnt(187,120),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(3,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(1,3,[pnt(187,120),pnt(187,187),pnt(263,263),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(2,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(2,1,[pnt(325,187),pnt(294,187),pnt(263,148),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(1,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(2,2,[pnt(325,187),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(4,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(2,3,[pnt(325,187),pnt(263,187),pnt(187,263),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(3,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(3,1,[pnt(263,325),pnt(263,294),pnt(299,263),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(2,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(3,2,[pnt(263,325),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(1,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(3,3,[pnt(263,325),pnt(263,263),pnt(187,187),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(4,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(4,1,[pnt(120,263),pnt(153,263),pnt(187,299),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(3,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(4,2,[pnt(120,263),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-!,
	pfeil_eckpunkte(2,PPkt1,PPkt2,PPkt3,PPkt4).
richtg_eckpunkte(4,3,[pnt(120,263),pnt(187,263),pnt(263,187),PPkt1,PPkt2,PPkt3,PPkt4,PPkt1]):-
	pfeil_eckpunkte(1,PPkt1,PPkt2,PPkt3,PPkt4).

kreuzg_background(Win):-!,
	PICTURE = pict_Load("KREUZG.BMP"),
	_RCT=win_GetClientRect(Win),
	pict_GetSize(PICTURE,X,Y,_),
	pict_Draw(Win,PICTURE,_RCT,rct(0,0,X,Y),rop_SrcCopy),
	pict_Destroy(PICTURE),!.

kreuzg_vzeichen(_,[]):-!.

kreuzg_vzeichen(Win,Positionen):-!,
	HAUPTSTR=pict_Load("HAUPTSTR.BMP"),
	NEBENSTR=pict_Load("NEBENSTR.BMP"),
	kreuzg_vzeichen_rek(1,Win,Positionen,HAUPTSTR,NEBENSTR),
	pict_Destroy(HAUPTSTR),
	pict_Destroy(NEBENSTR).

kreuzg_vzeichen_rek(5,_,_,_,_):-!.
kreuzg_vzeichen_rek(Zaehl,Win,Positionen,HAUPTSTR,NEBENSTR):-
	element(Zaehl,Positionen),!,
	find_loc_vz(Zaehl,X1,Y1,X2,Y2),
	pict_Draw(Win,HAUPTSTR,rct(X1,Y1,X2,Y2),rct(0,0,29,29),rop_SrcCopy),
	Zaehl1=Zaehl+1,
	kreuzg_vzeichen_rek(Zaehl1,Win,Positionen,HAUPTSTR,NEBENSTR).
kreuzg_vzeichen_rek(Zaehl,Win,Positionen,HAUPTSTR,NEBENSTR):-
	find_loc_vz(Zaehl,X1,Y1,X2,Y2),
	find_rot_vz(Zaehl,Rotation),
	NEBENSTR1=pict_Rotate(NEBENSTR,Rotation),
	pict_Draw(Win,NEBENSTR1,rct(X1,Y1,X2,Y2),rct(0,0,29,29),rop_SrcCopy),
	pict_Destroy(NEBENSTR1),
	Zaehl1=Zaehl+1,
	kreuzg_vzeichen_rek(Zaehl1,Win,Positionen,HAUPTSTR,NEBENSTR).

erzeuge_graf_listen(Referenz,Reihenfolge,Frageliste,Rotliste,Gruenliste):-
	erzeuge_graf_listen_rek(1,Referenz,Reihenfolge,[],Frageliste,[],Rotliste,[],Gruenliste).

erzeuge_graf_listen_rek(_,[],[],Frageliste,Frageliste,Rotliste,Rotliste,Gruenliste,Gruenliste):-!.
erzeuge_graf_listen_rek(Zaehl,[Ref|RefRest],[Ref|ReiRest],Flist,Frageliste,Rlist,Rotliste,Glist,Gruenliste):-
	Ref<>0,
	Zaehl1=Zaehl+1,
	erzeuge_graf_listen_rek(Zaehl1,RefRest,ReiRest,Flist,Frageliste,Rlist,Rotliste,[Zaehl|Glist],Gruenliste),!.
erzeuge_graf_listen_rek(Zaehl,[Ref|RefRest],[Ref|ReiRest],Flist,Frageliste,Rlist,Rotliste,Glist,Gruenliste):-
	Ref=0,
	Zaehl1=Zaehl+1,
	erzeuge_graf_listen_rek(Zaehl1,RefRest,ReiRest,Flist,Frageliste,Rlist,Rotliste,Glist,Gruenliste),!.
erzeuge_graf_listen_rek(Zaehl,[0|RefRest],[Rei|ReiRest],Flist,Frageliste,Rlist,Rotliste,Glist,Gruenliste):-
	Rei<>0,!,
	Zaehl1=Zaehl+1,
	erzeuge_graf_listen_rek(Zaehl1,RefRest,ReiRest,[Zaehl|Flist],Frageliste,Rlist,Rotliste,Glist,Gruenliste).
erzeuge_graf_listen_rek(Zaehl,[Ref|RefRest],[0|ReiRest],Flist,Frageliste,Rlist,Rotliste,Glist,Gruenliste):-
	Ref<>0,!,
	Zaehl1=Zaehl+1,
	erzeuge_graf_listen_rek(Zaehl1,RefRest,ReiRest,[Zaehl|Flist],Frageliste,Rlist,Rotliste,Glist,Gruenliste).
erzeuge_graf_listen_rek(Zaehl,[_|RefRest],[_|ReiRest],Flist,Frageliste,Rlist,Rotliste,Glist,Gruenliste):-
	Zaehl1=Zaehl+1,
	erzeuge_graf_listen_rek(Zaehl1,RefRest,ReiRest,Flist,Frageliste,[Zaehl|Rlist],Rotliste,Glist,Gruenliste).

kreuzg_autos(Win,Richtungen,Reihenfolge,Frageliste,Rotliste,Gruenliste):-
	finde_elem(1,Reihenfolge,_,Durchlauf1),
	finde_elem(2,Reihenfolge,_,Durchlauf2),
	finde_elem(3,Reihenfolge,_,Durchlauf3),
	retractall(win_content(Win,_)),
	kreuzg_autos_start(Win,Richtungen,Gruenliste,Rotliste,Frageliste),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT0=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT0),windows),
	pict_destroy(KREUZGCONT0),
	delay(Durchlauf1,200),
	kreuzg_autos_richtg(Win,1,Richtungen,color_Black),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf1,Durchlauf1,Gruenliste,Rotliste,2),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT11=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT11),windows),
	pict_destroy(KREUZGCONT11),
	delay(Durchlauf1,75),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf1,Durchlauf1,Gruenliste,Rotliste,3),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT12=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT12),windows),
	pict_destroy(KREUZGCONT12),
	delay(Durchlauf1,60),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf1,[],Gruenliste,Rotliste,4),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT13=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT13),windows),
	pict_destroy(KREUZGCONT13),
	delay(Durchlauf2,75),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf2,Durchlauf2,Gruenliste,Rotliste,2),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT21=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT21),windows),
	pict_destroy(KREUZGCONT21),
	delay(Durchlauf2,75),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf2,Durchlauf2,Gruenliste,Rotliste,3),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT22=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT22),windows),
	pict_destroy(KREUZGCONT22),
	delay(Durchlauf2,60),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf2,[],Gruenliste,Rotliste,4),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT23=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT23),windows),
	pict_destroy(KREUZGCONT23),
	delay(Durchlauf3,75),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf3,Durchlauf3,Gruenliste,Rotliste,2),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT31=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT31),windows),
	pict_destroy(KREUZGCONT31),
	delay(Durchlauf3,75),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf3,Durchlauf3,Gruenliste,Rotliste,3),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT32=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT32),windows),
	delay(Durchlauf3,60),
	kreuzg_autos_rek(Win,Richtungen,Durchlauf3,[],Gruenliste,Rotliste,4),
	retractall(win_content(Win,_)),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT33=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT33),windows).
	

kreuzg_autos_start(Win,Richtungen,Gruenliste,Rotliste,Frageliste):-	
	kreuzg_autos_start_rek(Win,1,Gruenliste),
	kreuzg_autos_start_rek(Win,2,Rotliste),
	kreuzg_autos_start_rek(Win,3,Frageliste),
	kreuzg_autos_richtg(Win,1,Richtungen,color_Yellow).

kreuzg_autos_richtg(_,_,[],_):-!.
kreuzg_autos_richtg(Win,Zaehl,[0|Rest],PenColor):-!,
	Zaehl1=Zaehl+1,
	kreuzg_autos_richtg(Win,Zaehl1,Rest,PenColor).
kreuzg_autos_richtg(Win,Zaehl,[Richtg|Rest],PenColor):-!,
	richtg_eckpunkte(Zaehl,Richtg,Punkte),
	win_SetPen(Win,pen(5,ps_Solid,PenColor)),
	draw_Polyline(Win,Punkte),
	Zaehl1=Zaehl+1,
	kreuzg_autos_richtg(Win,Zaehl1,Rest,PenColor).


kreuzg_autos_start_rek(_,_,[]):-!.
kreuzg_autos_start_rek(Win,1,[Pos|Rest]):-!,
	grafik_position(Pos,1,1,PosX,PosY,Rotation,BildNr),
	lade_bild(1,BildNr,Bild),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_Draw(Win,Bild1,pnt(PosX,PosY),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_start_rek(Win,1,Rest).
kreuzg_autos_start_rek(Win,2,[Pos|Rest]):-!,
	grafik_position(Pos,1,1,PosX,PosY,Rotation,BildNr),
	lade_bild(2,BildNr,Bild),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_Draw(Win,Bild1,pnt(PosX,PosY),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_start_rek(Win,2,Rest).
kreuzg_autos_start_rek(Win,3,[Pos|Rest]):-
	grafik_position(Pos,1,1,PosX,PosY,Rotation,BildNr),
	lade_bild(3,BildNr,Bild),
	grafik_position(Pos,1,1,PosX,PosY,Rotation,BildNr),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_Draw(Win,Bild1,pnt(PosX,PosY),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_start_rek(Win,3,Rest).

kreuzg_autos_rek(_,_,[],[],_,_,_):-!.
kreuzg_autos_rek(Win,Richtungen,[Pos|Rest],Positionen,Gruenliste,Rotliste,Fortschritt):-
	hole_elem(Richtungen,Pos,Richtg),
	element(Pos,Gruenliste),!,
	Fortschritt1=Fortschritt-1,
	grafik_position(Pos,Richtg,Fortschritt1,PosX,PosY,Rotation,BildNr),
	lade_bild(1,BildNr,Bild),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_GetSize(Bild1,X2,Y2,_),
	PosX2=PosX+X2,
	PosY2=PosY+Y2,
	pict_Draw(Win,Bild1,rct(PosX,PosY,PosX2,PosY2),rct(0,0,1,1),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_rek(Win,Richtungen,Rest,Positionen,Gruenliste,Rotliste,Fortschritt).
kreuzg_autos_rek(Win,Richtungen,[Pos|Rest],Positionen,Gruenliste,Rotliste,Fortschritt):-
	hole_elem(Richtungen,Pos,Richtg),
	element(Pos,Rotliste),!,
	Fortschritt1=Fortschritt-1,
	grafik_position(Pos,Richtg,Fortschritt1,PosX,PosY,Rotation,BildNr),
	lade_bild(1,BildNr,Bild),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_GetSize(Bild1,X2,Y2,_),
	PosX2=PosX+X2,
	PosY2=PosY+Y2,
	pict_Draw(Win,Bild1,rct(PosX,PosY,PosX2,PosY2),rct(0,0,1,1),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_rek(Win,Richtungen,Rest,Positionen,Gruenliste,Rotliste,Fortschritt).
kreuzg_autos_rek(Win,Richtungen,[_|Rest],Positionen,Gruenliste,Rotliste,Fortschritt):-!,
	kreuzg_autos_rek(Win,Richtungen,Rest,Positionen,Gruenliste,Rotliste,Fortschritt).
kreuzg_autos_rek(Win,Richtungen,[],[Pos|Rest],Gruenliste,Rotliste,Fortschritt):-
	hole_elem(Richtungen,Pos,Richtg),
	element(Pos,Gruenliste),!,
	grafik_position(Pos,Richtg,Fortschritt,PosX,PosY,Rotation,BildNr),
	lade_bild(1,BildNr,Bild),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_Draw(Win,Bild1,pnt(PosX,PosY),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_rek(Win,Richtungen,[],Rest,Gruenliste,Rotliste,Fortschritt).
kreuzg_autos_rek(Win,Richtungen,[],[Pos|Rest],Gruenliste,Rotliste,Fortschritt):-
	hole_elem(Richtungen,Pos,Richtg),
	element(Pos,Rotliste),!,
	grafik_position(Pos,Richtg,Fortschritt,PosX,PosY,Rotation,BildNr),
	lade_bild(2,BildNr,Bild),
	Bild1=pict_Rotate(Bild,Rotation),
	pict_Draw(Win,Bild1,pnt(PosX,PosY),rop_SrcCopy),
	pict_Destroy(Bild),
	pict_Destroy(Bild1),
	kreuzg_autos_rek(Win,Richtungen,[],Rest,Gruenliste,Rotliste,Fortschritt).
kreuzg_autos_rek(Win,Richtungen,[],[_|Rest],Gruenliste,Rotliste,Fortschritt):-
	kreuzg_autos_rek(Win,Richtungen,[],Rest,Gruenliste,Rotliste,Fortschritt).
	
lade_bild(1,1,Bild):-!,
	Bild=pict_Load("GRUEN-G.BMP").
lade_bild(1,2,Bild):-!,
	Bild=pict_Load("GRUEN-S.BMP").
lade_bild(2,1,Bild):-!,
	Bild=pict_Load("ROT-G.BMP").
lade_bild(2,2,Bild):-!,
	Bild=pict_Load("ROT-S.BMP").
lade_bild(_,_,Bild):-
	Bild=pict_Load("FRAGE.BMP").

%BEGIN_WIN Task Window
/***************************************************************************
		Event handling for Task Window
***************************************************************************/

predicates

  task_win_eh : EHANDLER

constants

%BEGIN Task Window, CreateParms, 16:41:00-9.1.2000, Code automatically updated!
  task_win_Flags = [wsf_SizeBorder,wsf_TitleBar,wsf_Close,wsf_Maximize,wsf_Minimize,wsf_ClipSiblings,wsf_ClipChildren]
  task_win_Menu  = res_menu(id_menu)
  task_win_Title = "Testumgebung Vorfahrtsregeln"
  task_win_Help  = idh_inhalt
%END Task Window, CreateParms

clauses

%BEGIN Task Window, e_Create
  task_win_eh(_Win,e_Create(_),0):-
%BEGIN Task Window, InitControls, 16:41:00-9.1.2000, Code automatically updated!
%END Task Window, InitControls
%BEGIN Task Window, ToolbarCreate, 16:41:00-9.1.2000, Code automatically updated!
%END Task Window, ToolbarCreate
	assert(run_num(1),lauf),
	win_kreuzg_Create(_Win,Kreuzg),
	assert(window_handle("Kreuzg",Kreuzg),windows),
	msg_Create(1000,"Kontrolle"),
	MsgWin=msg_GetWin(),
	edit_SetReadOnly(MsgWin,b_true),
	win_Move(MsgWin,rct(600,180,1000,700)),
	win_SetState(MsgWin,[wsf_visible]),
	assert(window_handle("Task Window", _Win),windows),
	win_SetState(_Win,[wsf_Maximized, wsf_visible]),
  write("Testumgebung zum KI-Praktikum \n\n\"Vorfahrtsregeln\"\n\n Version 0.5straight\n"),
  write("Copyright (c) 2000 Torsten Kurbad \nCopyright (c) Mai/Juni 2000 Robert Will\nCopyright (c) Mai 2003 Thomas Volkert & Denny Duphorn\n"),
  write("fr das Fachgebiet KI \nder Technischen Universitt Ilmenau\n\n"),
  write("Infos und Anleitung: \n"),
  write("www.theoinf.tu-ilmenau.de/ki/ki/ausbildung.html\n"),
  write("\n---\n\n"),
	existfile("Vorfahrt.ini"),!,		% falls dies failt...
	consult("Vorfahrt.ini",modus),
	dlg_Einstellungen_Create(_Win),
ifdef use_message
enddef
	!.
  task_win_eh(_Win,e_Create(_),0):-		% .. hier weiter.
  	assert(run_Modus(rvl),modus),
	dlg_Einstellungen_Create(_Win).

%END Task Window, e_Create

%END_WIN Task Window

/***************************************************************************
		Invoking on-line Help
***************************************************************************/

  project_ShowHelpContext(HelpTopic):-
  	vpi_ShowHelpContext("vorfahrt.hlp",HelpTopic).

/***************************************************************************
			Main Goal
***************************************************************************/

run:-
ifdef use_mdi
  vpi_SetAttrVal(attr_win_mdi,b_true),
enddef
ifdef ws_win
  ifdef use_3dctrl
    vpi_SetAttrVal(attr_win_3dcontrols,b_true),
  enddef
enddef  
  vpi_Init(task_win_Flags,task_win_eh,task_win_Menu,"vorfahrt",task_win_Title).

goal run.

%BEGIN_WIN Kreuzg
/**************************************************************************
        Creation and event handling for window: Kreuzg
**************************************************************************/

constants
%BEGIN Kreuzg, CreateParms, 16:42:54-9.1.2000, Code automatically updated!
  win_kreuzg_WinType = w_TopLevel
  win_kreuzg_Flags = [wsf_TitleBar]
  win_kreuzg_RCT = rct(20,40,470,490)
  win_kreuzg_Menu = no_menu
  win_kreuzg_Title = "Kreuzung"
  win_kreuzg_Help = idh_inhalt
%END Kreuzg, CreateParms

predicates

  win_kreuzg_eh : EHANDLER

clauses

  win_kreuzg_Create(_Parent,Handle):-
	Handle=win_Create(win_kreuzg_WinType,win_kreuzg_RCT,win_kreuzg_Title,
		   win_kreuzg_Menu,_Parent,win_kreuzg_Flags,win_kreuzg_eh,0).

%BEGIN Kreuzg, e_Create
  win_kreuzg_eh(_Win,e_Create(_),0):-!,
%BEGIN Kreuzg, InitControls, 16:42:54-9.1.2000, Code automatically updated!
%END Kreuzg, InitControls
%BEGIN Kreuzg, ToolbarCreate, 16:42:54-9.1.2000, Code automatically updated!
%END Kreuzg, ToolbarCreate
	!.
%END Kreuzg, e_Create
%MARK Kreuzg, new events

%BEGIN Kreuzg, e_Update
  win_kreuzg_eh(_Win,e_Update(_UpdateRct),0):-
	kreuzg_redraw(_Win),
	!.

/*  win_kreuzg_eh(_Win,e_Update(_UpdateRct),0):-
	kreuzg_background(_Win),
	!.
*/%END Kreuzg, e_Update

%BEGIN Kreuzg, e_Size
  win_kreuzg_eh(_Win,e_Size(_Width,_Height),0):-!,
ifdef use_tbar
	toolbar_Resize(_Win),
enddef
	!.
%END Kreuzg, e_Size

%BEGIN Kreuzg, e_Menu, Parent window 
  win_kreuzg_eh(Win,e_Menu(ID,CAS),0):-!,
	PARENT = win_GetParent(Win),
	win_SendEvent(PARENT,e_Menu(ID,CAS)),
	!.
%END Kreuzg, e_Menu, Parent window

%END_WIN Kreuzg


%BEGIN_DLG Einstellungen
/**************************************************************************
	Creation and event handling for dialog: Einstellungen
**************************************************************************/

constants

%BEGIN Einstellungen, CreateParms, 13:31:36-6.6.2000, Code automatically updated!
  dlg_einstellungen_ResID = idd_control_settings
  dlg_einstellungen_DlgType = wd_Modeless
  dlg_einstellungen_Help = idh_contents
%END Einstellungen, CreateParms

predicates

  dlg_einstellungen_eh : EHANDLER

clauses

  dlg_einstellungen_Create(Parent):-

%MARK Einstellungen, new variables

	dialog_CreateModeless(Parent,dlg_einstellungen_ResID,"",
  		[
%BEGIN Einstellungen, ControlList, 13:31:36-6.6.2000, Code automatically updated!
		df(idc_num_runs_val,editstr("1",[]),nopr),
		df(idc_einstellungen_1,listbox(["Rechts vor Links","Abbiegende Hauptstrae","Haupt- und Nebenstrassen"],[0]),nopr)
%END Einstellungen, ControlList
		],
		dlg_einstellungen_eh,0).

%BEGIN Einstellungen, idc_end _CtlInfo
  dlg_einstellungen_eh(_Win,e_Control(idc_ende,_CtrlType,_CtrlWin,_CtrlInfo),0):-!,
  	window_handle("Task Window", TWin),
  	win_SendEvent(TWin,e_CloseRequest),
	win_Destroy(TWin),!.
%END Einstellungen, idc_ok _CtlInfo
%MARK Einstellungen, new events

%BEGIN Einstellungen, e_Create
  dlg_einstellungen_eh(_Win,e_Create(_CreationData),0):-
	% Werte in Kontrollfelder eintragen
	run_num(Anzahl),
	str_int(AnzStr,Anzahl),
	Edit=win_GetCtlHandle(_Win,idc_num_runs_val),
	win_SetText(Edit,AnzStr),    % Anzahl
	run_modus(Modus),
	List=win_GetCtlHandle(_Win,idc_einstellungen_1),
	lbox_SetSel(List,Modus,b_true),  % Modus
	!.
%END Einstellungen, e_Create

%BEGIN idc_einstellungen_1 selchanged
% Modus ndern
  dlg_einstellungen_eh(_Win,e_Control(idc_einstellungen_1,_CtrlType,_CtrlWin,selchanged),0):-!,
  	List=win_GetCtlHandle(_Win,idc_einstellungen_1),
	retractall(run_modus(_),modus),
	Modus=lbox_GetSelIndex(List),
	assert(run_modus(Modus),modus),
	save("Vorfahrt.ini",modus),
	!.

%BEGIN idc_num_runs_val modified
% Anzahl ndern
  dlg_einstellungen_eh(_Win,e_Control(idc_num_runs_val,_CtrlType,_CtrlWin,modified),0):-!,
	Edit=win_GetCtlHandle(_Win,idc_num_runs_val),
	Anz_Str=win_GetText(Edit),
	str_int(Anz_Str,Anzahl),
	retractall(run_num(_),lauf),
	assert(run_num(Anzahl),lauf),
	!.

%BEGIN Einstellungen, idc_ok _CtlInfo
  dlg_einstellungen_eh(_Win,e_Control(idc_ok,_CtrlType,_CtrlWin,_CtlInfo),0):-!,
	% Starten.
  	run_modus(Modus),      
  	run_num(Anzahl),
  	start(Modus,Anzahl),
	!.
%END Einstellungen, idc_ok _CtlInfo

%BEGIN id_hilfe
  dlg_einstellungen_eh(_Win,e_Control(idc_hilfe,_CtrlType,_CtrlWin,_CtlInfo),0):-!,
  	vpi_ShowHelp("vorfahrt.hlp"),
	!.

  dlg_einstellungen_eh(_,_,_):-!,fail.

%END_DLG Einstellungen

clauses

grafik(_,_,_):-
	run_num(Runs),
	Runs>1,!.

grafik(Testsit,Referenz,Reihenfolge):-
	finde_hstr(Testsit,Hauptstrliste),
	window_handle("Kreuzg",Win),
	win_BringToTop(Win),
	garb_coll(Win),
	retractall(win_content(Win,_),windows),
	kreuzg_background(Win),
	kreuzg_vzeichen(Win,Hauptstrliste),
	_RCT=win_GetClientRect(Win),
	KREUZGCONT=pict_GetFromWin(Win,_RCT),
	assert(win_content(Win,KREUZGCONT),windows),
	ersetze_alle_richtg(Testsit,Richtungen),
	erzeuge_graf_listen(Referenz,Reihenfolge,Frageliste,Rotliste,Gruenliste),
	kreuzg_autos(Win,Richtungen,Reihenfolge,Frageliste,Rotliste,Gruenliste),
	MsgWin=msg_GetWin(),
	win_BringToTop(MsgWin),!.
	




